package ub.pfg.preyrobot.activities;

import android.app.Activity;
import ub.pfg.preyrobot.R;
import ub.pfg.preyrobot.vision.ColorTracker;
import ub.pfg.preyrobot.vision.ORBTracker;
import ub.pfg.preyrobot.vision.OpticalFlowTracker;
import ub.pfg.preyrobot.vision.Tracker;
import android.os.Bundle;
import android.util.Log;
import android.view.WindowManager;
import org.opencv.android.BaseLoaderCallback;
import org.opencv.android.LoaderCallbackInterface;
import org.opencv.android.OpenCVLoader;
import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewListener;
import org.opencv.core.Mat;

public class CameraActivity extends Activity implements CvCameraViewListener {
	private static final String  TAG = "CameraActivity";
	
	private CameraBridgeViewBase   	openCvCameraView = null;
	private Tracker 				tracker = null;
	private Mat 					m = null;
    
    private BaseLoaderCallback mLoaderCallback = new BaseLoaderCallback(this) {
        @Override
        public void onManagerConnected(int status) {
            switch (status) {
                case LoaderCallbackInterface.SUCCESS:
                {
                    Log.i( TAG, "OpenCV loaded successfully" );
                    initData();
                } break;
                default:
                {
                    super.onManagerConnected(status);
                } break;
            }
        }	
    };
    
    private void initData() {
    	tracker.initDetector();
        openCvCameraView.enableView();
        openCvCameraView.enableFpsMeter();
	}

    @Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		setContentView( R.layout.camera_activity );
		
		openCvCameraView = (CameraBridgeViewBase) findViewById( R.id.surface_view );
		openCvCameraView.setCvCameraViewListener(this);
		
		Bundle extras = getIntent().getExtras();
		String method = extras.getString( "method" );

		if( method.equals( "flow" ) ) {
			tracker = new OpticalFlowTracker();
		} else if( method.equals( "orb" )) {
			tracker = new ORBTracker();
		} else {
			tracker = new ColorTracker();
		}
	}

	@Override
    public void onPause() {
        if (openCvCameraView != null)
            openCvCameraView.disableView();
        super.onPause();
    }

    @Override
    public void onResume() {
        super.onResume();
        OpenCVLoader.initAsync( OpenCVLoader.OPENCV_VERSION_2_4_3, this, mLoaderCallback );
    }
    
	@Override
	public void onCameraViewStarted( int width, int height ) {
		tracker.start();
	}

	@Override
	public void onCameraViewStopped() {
	}

	@Override
	public Mat onCameraFrame( Mat inputFrame ) {
		m = tracker.processFrame( inputFrame );
		return m;
	}
	
	@Override
    public void onDestroy() {
        super.onDestroy();
        if (openCvCameraView != null) {
            openCvCameraView.disableView();
        }
        if( m != null ) {
        	m.release();
        }
        tracker.stop();
    }
}
