package ub.pfg.preyrobot.activities;

import java.lang.reflect.InvocationTargetException;

import ub.pfg.preyrobot.comunications.BluetoothCom;
import ub.pfg.preyrobot.R;
import android.os.Bundle;
import android.app.Activity;
import android.content.Intent;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.RadioGroup;

public class MainActivity extends Activity {
	private static final String TAG = "MainActivity";
	private static final String DAVID_ADDRESS = "00:16:53:08:7B:94";
	
	private BluetoothCom bluetoothCom = null;
	
	private Intent intent;
	private Button btnConnect;
	private Button btnStart;

	private RadioGroup group;
	
	@Override
	protected void onCreate( Bundle savedInstanceState ) {
		super.onCreate( savedInstanceState );
		setContentView( R.layout.activity_main );
		
		intent = new Intent( MainActivity.this, CameraActivity.class );
		
		bluetoothCom = BluetoothCom.getInstance();
		
		group = (RadioGroup) findViewById( R.id.option_method );
		btnConnect = (Button) findViewById( R.id.connect_button );
		btnStart = (Button) findViewById( R.id.start_btn );
		
		btnConnect.setEnabled( false );
		btnConnect.setText( "Wait..." );
		
		btnConnect.setOnClickListener( new View.OnClickListener() {
			@Override
			public void onClick( View v ) {
				startConnection();
				//startActivity( intent );
			}
		});
		
		btnStart.setOnClickListener( new View.OnClickListener() {
			@Override
			public void onClick( View v ) {
				int radioButtonID = group.getCheckedRadioButtonId();
				View radioButton = group.findViewById( radioButtonID );
				int idx = group.indexOfChild( radioButton );
				
				String method = null;
				if( idx == 0 ) {
					method = "flow";
				} else if( idx == 1 ) {
					method = "orb";
				} else {
					method = "color";
				}
				if( bluetoothCom.isConnected() ) {
					intent.putExtra( "method", method );
					startActivity( intent );
				} else {
					intent.putExtra( "method", method );
					startActivity( intent );
				}
			}
		});
		
	}
	
	@Override
    public void onStart() {
        super.onStart();
        Log.e(TAG, "++ ON START ++");
        
        bluetoothCom.enableComunication( false, true );
        
        btnConnect.setEnabled( true );
        btnConnect.setText( "Connect" );
    }
	
	private void startConnection() {
		if( !bluetoothCom.isConnected() ) {
			btnConnect.setText( "Connecting..." );
			btnConnect.setEnabled( false );
			
			try {
				// Attempt to connect to the device
				bluetoothCom.connect( DAVID_ADDRESS );
			}
			catch (IllegalArgumentException e) { }
			catch (NoSuchMethodException e) { }
			catch (IllegalAccessException e) { }
			catch (InvocationTargetException e) { }

			if( !bluetoothCom.isConnected() ) {
				btnConnect.setText( "No connections available" );
			} else {
				btnConnect.setText( "Disconnect" );
				btnConnect.setEnabled( true );
				
				//startActivity( intent );
			}
			
		} else {
			btnConnect.setText( "Disconnecting..." );
			btnConnect.setEnabled( false );
			bluetoothCom.stopConection();
			btnConnect.setText( "Connect" );
			btnConnect.setEnabled( true );
		}
	}
	
	@Override
    public void onDestroy() {
        super.onDestroy();
        Log.e(TAG, "--- ON DESTROY ---");
        bluetoothCom.stopConection();
        bluetoothCom.turnBluetoothOff();
    }
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_main, menu);
		return true;
	}

}
