package ub.pfg.preyrobot.comunications;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class BluetoothCom {
	private static final String TAG = "BluetoothCom";
	
	private RobotUtils RU = new RobotUtils();
	
	private static final int ERROR = -1;
	private static final int DISCONNECTED = 0;
	private static final int CONNECTED = 1;
	private static final int WRITING = 2;
	
	private static final int BUFFER_SIZE = 128; // 64
	
	private BluetoothAdapter bluetoothAdapter = null;
    private BluetoothSocket socket = null;
    private OutputStream out = null;
    private InputStream in = null;
    private int state = DISCONNECTED;
    
    private static BluetoothCom instance;
    
    private Handler mHandler;
    private ConnectedThread readingThread;
    private byte[] buffer;
    private boolean exit = false;
    
    private boolean read;
    private boolean write;
    
    public static synchronized BluetoothCom getInstance() {
    	if( instance == null ) {
    		instance = new BluetoothCom();
    	}
    	return instance;
    }
    
    public void enableComunication( boolean read, boolean write ) {
    	this.read = read;
    	this.write = write;
    	
    	// Get local Bluetooth adapter
		bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		
		// If the adapter is null, then Bluetooth is not supported
        if (bluetoothAdapter == null) {
            //Toast.makeText(this, "Bluetooth is not available", Toast.LENGTH_LONG).show();
            //finish();
        	state = ERROR;
            return;
        }
        
    	if (!bluetoothAdapter.isEnabled()) {
        	bluetoothAdapter.enable();
        }
        while( !bluetoothAdapter.isEnabled() ) {}
    }
	
	public synchronized void connect( String address ) throws NoSuchMethodException,
				IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		
		if( state == DISCONNECTED ) {
			// Get the BluetoothDevice object
	        BluetoothDevice device = bluetoothAdapter.getRemoteDevice( address );
	        Log.e(TAG, "connect to: " + device);
			
			Method m = device.getClass().getMethod("createRfcommSocket", new Class[] { int.class });
			socket = (BluetoothSocket)m.invoke(device, Integer.valueOf(1));
		
			try {
	            socket.connect();
	            state = CONNECTED;
	            if( write ) {
	            	out = socket.getOutputStream();
	            } else if( read ) {
	            	in = socket.getInputStream();
	            	buffer = new byte[BUFFER_SIZE];
		            readingThread = new ConnectedThread();
	            }
	        } catch (IOException e) {
	        	Log.e(TAG, "Error in connection");
	            try {
					socket.close(); // Close the socket
				} catch (IOException e1) {
					Log.e(TAG, "Error closing socket");
				}
				state = DISCONNECTED;
	            return;
	        }
		}
	}
	
	public boolean isConnected() {
		if( state == CONNECTED ) {
			return true;
		}
		return false;
	}
	
	public void write( byte[] data ) {
		if( state == CONNECTED ) {
			state = WRITING;
			try {
				out.write( data );
				state = CONNECTED;
				Log.e(TAG, "SENDING BYTES TO ROBOT");
			} catch (IOException e) {
				state = DISCONNECTED;
				Log.e(TAG, "Exception during write", e);
			}
		}
	}
	
	private String decodeMessage( int bytes ) {
		String m = new String();
		int length = (int)buffer[6];
		
		if( length > 0 ) {
			length += 6; // tenemos en cuenta los bytes que preceden al mensaje
			
			for( int i = 7; i < length; i++ ) {
				m += (char)buffer[i];
			}
			return m;
		}
		return null;
	}

	public synchronized void stopConection() {
		if( socket != null && state == CONNECTED ) {
			try {
				if( readingThread != null ) {
					exit = true;
					try {
						readingThread.join();
					} catch (InterruptedException e) {
						Log.i( TAG, "ERROR EN EL JOIN" );
						e.printStackTrace();
					}
				}
				socket.close();
				state = DISCONNECTED;
			} catch (IOException e) { }
		}
		state = DISCONNECTED;
	}
	
	public void turnBluetoothOff() {
		bluetoothAdapter.disable();
	}
	
	public void setHandler( Handler handler ) {
		if( read ) {
			mHandler = handler;
			readingThread.start();
		}
	}
	
    //This thread runs during a connection with a remote device.
    //It handles all incoming and outgoing transmissions.
    private class ConnectedThread extends Thread {
                
        public void run() {
            Log.i(TAG, "BEGIN mConnectedThread");
            int bytes;

            // Keep listening to the InputStream while connected
            while ( !exit ) {
                try {
                    // Read from the InputStream
                	out.write( RU.read_mailbox );
                    bytes = in.read( buffer );
                    String m = decodeMessage( bytes );
                    Message msg = new Message();
                    msg.obj = m;
                    mHandler.sendMessage( msg );
                } catch (IOException e) {
                    Log.e(TAG, "disconnected", e);
                    stopConection();
                    break;
                }
            }
        }
    }

}
