package ub.pfg.preyrobot.vision;

import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.MatOfKeyPoint;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;
import android.util.Log;

public class ColorTracker extends Tracker {
	private static final String TAG = "ColorTracker";
	
	// Color naranja
	/*private final Scalar UPPER_THRESHOLD = new Scalar( 20, 155, 255 );
	private final Scalar LOWER_THRESHOLD = new Scalar( 0, 105, 135 );*/
	
	// Amarillo funciona pero detecta rojo y verde tambien
	//private final Scalar UPPER_THRESHOLD = new Scalar( 140, 300, 360 );
	//private final Scalar LOWER_THRESHOLD = new Scalar( 40, 140, 140 );
	
	// Amarillo filtra mejor los otros colores
	private final Scalar UPPER_THRESHOLD = new Scalar( 65, 360, 360 ); // 100 y 80
	private final Scalar LOWER_THRESHOLD = new Scalar( 55, 120, 160 );
	
	private final int FACTOR = 4;
	
	private Mat hsv, binary, aux;

	private Size reduced_size;
	
	public void initDetector() {
		super.initDetector();
		
		hsv = new Mat();
		binary = new Mat();
		aux = new Mat();
		
		reduced_size = new Size( WIDTH/FACTOR, HEIGHT/FACTOR );
	}
	
	public Mat processFrame( Mat rgba ) {
		Imgproc.cvtColor( rgba, aux, Imgproc.COLOR_RGB2HSV_FULL );
		
		Imgproc.resize( aux, hsv, reduced_size );
		
		Core.inRange( hsv, LOWER_THRESHOLD, UPPER_THRESHOLD, hsv );
		
		Core.bitwise_or( hsv, hsv, binary );
		
		computePosition();
		if( !lost ) {
			Core.circle( rgba, currentPosition, 10, new Scalar( 255, 0, 0 ), -1, 8, 0 );
		}
		
		computeDirectionAndFactor();
		RI.sendProportionalMovement( direction, prop_factor );
		
		updatePrevPosition();
		
		return rgba;
	}
	
	protected void computePosition() {
		int x, y, countX, countY;
		x = 0; y = 0; countX = 0; countY = 0;
		
		for( int i = 0; i < binary.rows(); i++ ) {
			for( int j = 0; j < binary.cols(); j++ ) {
				double a = binary.get( i, j )[0];
				
				if( a > 50 ) {
					x += j;
					y += i;
					countX++;
					countY++;
				}
			}
		}
		if( countX != 0 && countY != 0) {
			currentPosition.x = (x/countX)*FACTOR;
			currentPosition.y = (y/countY)*FACTOR;
			lost = false;
		} else {
			currentPosition.x = 0;
			currentPosition.y = 0;
			//MatOfKeyPoint m = new MatOfKeyPoint();
			
			//m.fromList();
			lost = true;
		}
	}
	
	/*private Mat dibujarCirculo( Mat rgba ) {
		Mat circles = new Mat();
		// pre smooth
		
		// Apply the Hough Transform to find the circles
		Imgproc.HoughCircles( hsv, circles, Imgproc.CV_HOUGH_GRADIENT, 1, hsv.rows()/8 );
		
		if( circles.cols() > 0 ) {
			for( int i = 0; i < circles.cols(); i++ ) {
				double[] c = circles.get( 0, 0 );
				int radius = (int) Math.round( c[2] );
				Point center = new Point( Math.round( c[0] ), Math.round( c[1] ) );
				center.x *= FACTOR;
				center.y *= FACTOR;
				Core.circle( rgba, center, 3, new Scalar( 0, 255, 0 ), -1, 8, 0 );
				Core.circle( rgba, center, radius, new Scalar( 0, 255, 0 ), 3, 8, 0 );
			}
		}
		return rgba;
	}*/
	
	public void stop() {
		super.stop();
		
		if( hsv != null) {
			hsv.release();
			hsv = null;
		}
	}

}
