package ub.pfg.preyrobot.vision;

import java.util.ArrayList;
import java.util.List;
import org.opencv.core.Mat;
import org.opencv.core.MatOfDMatch;
import org.opencv.core.MatOfKeyPoint;
import org.opencv.core.Size;
import org.opencv.features2d.DMatch;
import org.opencv.features2d.DescriptorExtractor;
import org.opencv.features2d.DescriptorMatcher;
import org.opencv.features2d.FeatureDetector;
import org.opencv.features2d.Features2d;
import org.opencv.highgui.Highgui;

import android.os.Environment;
import android.util.Log;

public class FeatureMatcher {
	private static final String TAG = "FeatureMatcher";
	
	private static final double RATIO = 1.4;
	
	public FeatureDetector 		detector;
	public DescriptorExtractor 	extractor;
	public DescriptorMatcher 	matcher;
	
	private MatOfKeyPoint 		train_keypoints;
	private MatOfKeyPoint		query_keypoints;
	private Mat 				train_descriptor;
	private Mat 				query_descriptor;
	private MatOfDMatch 		matches;
	
	public Mat 					trainImage;
	public Mat	 				matching;
	
	private boolean				debug;
	
	private MatOfKeyPoint 		queryDes;
	private List<MatOfDMatch> 	matchesList;
	
	
	public FeatureMatcher() {
		train_keypoints = new MatOfKeyPoint();
		query_keypoints = new MatOfKeyPoint();
		train_descriptor = new Mat();
		query_descriptor = new Mat();
	    matches = new MatOfDMatch();
	    matching = new Mat();
	    trainImage = new Mat();
	    
	    queryDes = new MatOfKeyPoint();
	    matchesList = new ArrayList<MatOfDMatch>();
	    
	    detector = FeatureDetector.create( FeatureDetector.ORB ); // ORB
	    extractor = DescriptorExtractor.create( DescriptorExtractor.FREAK ); // FREAK / BRISK
	    matcher = DescriptorMatcher.create( DescriptorMatcher.BRUTEFORCE_HAMMING );
	}
	
	public void computePrevDescriptor( boolean saveImage ) {
		if( debug )
			Log.i( TAG, "COMPUTANDO EL DESCRIPTOR" );
		
		detector.detect( trainImage, train_keypoints );
		extractor.compute( trainImage, train_keypoints, train_descriptor );
		
		if( saveImage ) {
			Features2d.drawKeypoints( trainImage, train_keypoints, trainImage );
			if( Highgui.imwrite( Environment.getExternalStorageDirectory()+"/patron1.jpg", trainImage ) ) {
				//Log.i( TAG, "IMG GUARDADA" );
			}
		}
	}
	
	public boolean detectAndMatching( Mat current ) {
		detector.detect( current, query_keypoints );
		extractor.compute( current, query_keypoints, query_descriptor );
		
		if( query_descriptor.rows() >= 10 && query_descriptor.cols() >= 10 ) {
			matcher.match( train_descriptor, query_descriptor, matches );
			
			filterMatches( RATIO );
			if( matches.rows() >= 5 ) {
				Features2d.drawMatches( trainImage, train_keypoints, current, query_keypoints, matches, matching );
				if( debug )
					writeStatus();
				return true;
			}
		}
		return false;
	}
	
	public void filterMatches( double ratio ) {
		List<DMatch> matches_tmp = matches.toList();
		//double max_dist = 0;
		double min_dist = 100;
		int rowCount = matches_tmp.size();
		for (int i = 0; i < rowCount; i++) {
			double dist = matches_tmp.get(i).distance;
			//System.out.println("distance dist==" + dist);
			if (dist < min_dist)
				min_dist = dist;
			/*if (dist > max_dist)
				max_dist = dist;*/
		}
		
		//System.out.println("rowCount===" + rowCount + "max_dist= " + max_dist + "min_dist=" + min_dist);
		List<DMatch> good_matches = new ArrayList<DMatch>();
		double good_dist = ratio * min_dist; // 3 (a 2 detecta unas 300)
		for (int i = 0; i < rowCount; i++) {
			if (matches_tmp.get(i).distance < good_dist) {
			good_matches.add(matches_tmp.get(i));
			// Core.circle(Mat, points.get( matches.get(i).trainIdx ).pt, 8,
			// color, 1);
			//System.out.println("good_matches......." + matches_tmp.get(i).trainIdx);
			}
		}
		matches.fromList( good_matches );
		Log.i( TAG, "GOOD MATCHES: "+matches.size() );		
	}
	
	public boolean detectAndKnnMatching( Mat current ) {		
		detector.detect( current, query_keypoints );
		extractor.compute( current, query_keypoints, queryDes );
		
		if( queryDes.rows() >= 10 && queryDes.cols() >= 10 ) {
			matcher.knnMatch( queryDes, train_descriptor, matchesList, 10 );
			//writeStatusKnn();
			//MatOfDMatch matches = matchesList.get(0);
			//Features2d.drawMatches( trainImage, train_keypoints, current, query_keypoints, matches, matching );
			
			Size querySize = queryDes.size();
			if( querySize.width >= 10 ) {
				if( matchesList.size() > 10 ) {
					//for( int i = 0; i < matchesList.size(); i++ ) {
						//Features2d.drawMatches( trainImage, train_keypoints, current, queryDes, matchesList.get(i), matching );
					//}
					MatOfDMatch matches = matchesList.get(0);
					Features2d.drawMatches( trainImage, train_keypoints, current, query_keypoints, matches, matching );
					if( debug )
						writeStatusKnn();
					return true;
				}
			}
		}
		return false;
	}
	
	public void writeStatus() {
		Log.i( TAG, "KEYPOINTS-----------------------" );
		Log.i( TAG, "COLUMNAS: "+ query_keypoints.cols() );
		Log.i( TAG, "FILAS: "+ query_keypoints.rows() );
		
		Log.i( TAG, "QUERY DESCRIPTOR----------------" );
		Log.i( TAG, "COLUMNAS: "+ query_descriptor.cols() );
		Log.i( TAG, "FILAS: "+ query_descriptor.rows() );
		
		Log.i( TAG, "MATCHES---------------------" );
		Log.i( TAG, "COLUMNAS: "+ matches.cols() );
		Log.i( TAG, "FILAS: "+ matches.rows() );
	}
	
	public void writeStatusKnn() {
		Log.i( TAG, "KEYPOINTS-----------------------" );
		Log.i( TAG, "COLUMNAS: "+ query_keypoints.cols() );
		Log.i( TAG, "FILAS: "+ query_keypoints.rows() );
		
		Log.i( TAG, "QUERYDES------------------------" );
		Log.i( TAG, "SIZE: "+ queryDes.size() );
		
		Log.i( TAG, "MATCHESLIST---------------------" );
		Log.i( TAG, "COLUMNAS: "+ matchesList.size() );
	}
	
	public void enableDebug() {
		debug = true;
	}

	public void disableDebug() {
		debug = false;
	}
}
