package ub.pfg.preyrobot.vision;

import org.opencv.core.Mat;
import org.opencv.core.Point;
import org.opencv.core.Size;
import android.util.Log;
import ub.pfg.preyrobot.comunications.RobotInterface;

public class Tracker {
	private static final String TAG = "Tracker";
	
	// Size for images
	protected static final int 	WIDTH = 800,
								HEIGHT = 480;
	
	protected Size imgSize = new Size( WIDTH, HEIGHT );
	
	private static final int REFERENCE = HEIGHT/2;
	
	private static final int TOLERANCIA = HEIGHT/20;
	
	protected RobotInterface RI = null;
	
	protected boolean lost;
	
	protected Point prevPosition, currentPosition;
	
	protected byte direction = 'l';
	protected int prop_factor = 0;

	public void initDetector() {
		Log.i( TAG, "INIT DETECTOR" );
		RI = new RobotInterface();
		currentPosition = new Point( 0, 0 );
		prevPosition = new Point( 0, 0 );
	}
	
	public final void start() {
		RI.sendInit(); // send init to robot
		Log.i( "MSG", "INIT ENVIADO" );
	}
	
	public Mat processFrame( Mat rgba ) {
		return rgba;
	}
	
	protected void computePosition() {
	}
	
	protected void updatePrevPosition() {
		prevPosition.x = currentPosition.x;
		prevPosition.y = currentPosition.y;
	}
	
	protected void computeDirectionAndFactor() {
		int pos;
		
		pos = (int) currentPosition.y;
		
		if( lost ) {
			direction = (byte)'l';
		
		// (0,0) en la esquina superior derecha de la imagen
		} else {
			int x = (int) ((pos - REFERENCE)/10);
			Log.i( TAG, "POS: "+pos );
			Log.i( TAG, "REF: "+REFERENCE );
			
			if( pos > (REFERENCE + TOLERANCIA) ) { // left
				direction = (byte)'i';
				
			} else if( pos < (REFERENCE - TOLERANCIA) ) { // right
				direction = (byte)'d';
				x = x * (-1);
				
			} else {
				direction = (byte)'c';
				x = 0;
			}
			prop_factor = x*2;
		}
	}
	
	public void stop() {
		Log.i( TAG, "STOP DETECTOR" );
		if( RI != null ) {
			RI.stop();
			RI = null;
		}
	}
	
}
